@@ -50,6 +50,20 @@ string_to_guint32_with_default (const char *n, guint32 def)
         return z;
 }
 
+typedef struct {
+    const char *md_type;
+    xmlParserCtxt *xml_context;
+    GError **error;
+    CountFn count_fn;
+    PackageFn package_fn;
+    gpointer user_data;
+
+    Package *current_package;
+
+    gboolean want_text;
+    GString *text_buffer;
+} SAXContext;
+
 typedef enum {
     PRIMARY_PARSER_TOPLEVEL = 0,
     PRIMARY_PARSER_PACKAGE,
@@ -58,19 +72,12 @@ typedef enum {
 } PrimarySAXContextState;
 
 typedef struct {
-    xmlParserCtxt *xml_context;
+    SAXContext sctx;
+
     PrimarySAXContextState state;
-    GError **error;
-    CountFn count_fn;
-    PackageFn package_fn;
-    gpointer user_data;
 
-    Package *current_package;
     GSList **current_dep_list;
     PackageFile *current_file;
-
-    gboolean want_text;
-    GString *text_buffer;
 } PrimarySAXContext;
 
 static void
@@ -78,15 +85,17 @@ primary_parser_toplevel_start (PrimarySAXContext *ctx,
                                const char *name,
                                const char **attrs)
 {
+    SAXContext *sctx = &ctx->sctx;
+
     if (!strcmp (name, "package")) {
-        g_assert (ctx->current_package == NULL);
+        g_assert (sctx->current_package == NULL);
 
         ctx->state = PRIMARY_PARSER_PACKAGE;
 
-        ctx->current_package = package_new ();
+        sctx->current_package = package_new ();
     }
 
-    else if (ctx->count_fn && !strcmp (name, "metadata")) {
+    else if (sctx->count_fn && !strcmp (name, "metadata")) {
         int i;
         const char *attr;
         const char *value;
@@ -96,8 +105,8 @@ primary_parser_toplevel_start (PrimarySAXContext *ctx,
             value = attrs[++i];
 
             if (!strcmp (attr, "packages")) {
-                ctx->count_fn (string_to_guint32_with_default (value, 0),
-                               ctx->user_data);
+                sctx->count_fn (string_to_guint32_with_default (value, 0),
+                               sctx->user_data);
                 break;
             }
         }
@@ -129,14 +138,16 @@ primary_parser_package_start (PrimarySAXContext *ctx,
                               const char *name,
                               const char **attrs)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
     int i;
     const char *attr;
     const char *value;
 
     g_assert (p != NULL);
 
-    ctx->want_text = TRUE;
+    sctx->want_text = TRUE;
 
     if (!strcmp (name, "format")) {
         ctx->state = PRIMARY_PARSER_FORMAT;
@@ -200,7 +211,9 @@ primary_parser_format_start (PrimarySAXContext *ctx,
                              const char *name,
                              const char **attrs)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
     int i;
     const char *attr;
     const char *value;
@@ -221,16 +234,16 @@ primary_parser_format_start (PrimarySAXContext *ctx,
 
     else if (!strcmp (name, "rpm:provides")) {
         ctx->state = PRIMARY_PARSER_DEP;
-        ctx->current_dep_list = &ctx->current_package->provides;
+        ctx->current_dep_list = &sctx->current_package->provides;
     } else if (!strcmp (name, "rpm:requires")) {
         ctx->state = PRIMARY_PARSER_DEP;
-        ctx->current_dep_list = &ctx->current_package->requires;
+        ctx->current_dep_list = &sctx->current_package->requires;
     } else if (!strcmp (name, "rpm:obsoletes")) {
         ctx->state = PRIMARY_PARSER_DEP;
-        ctx->current_dep_list = &ctx->current_package->obsoletes;
+        ctx->current_dep_list = &sctx->current_package->obsoletes;
     } else if (!strcmp (name, "rpm:conflicts")) {
         ctx->state = PRIMARY_PARSER_DEP;
-        ctx->current_dep_list = &ctx->current_package->conflicts;
+        ctx->current_dep_list = &sctx->current_package->conflicts;
     }
 
     else if (!strcmp (name, "file")) {
@@ -252,6 +265,8 @@ primary_parser_dep_start (PrimarySAXContext *ctx,
                           const char *name,
                           const char **attrs)
 {
+    SAXContext *sctx = &ctx->sctx;
+
     const char *tmp_name = NULL;
     const char *tmp_version = NULL;
     const char *tmp_release = NULL;
@@ -288,7 +303,7 @@ primary_parser_dep_start (PrimarySAXContext *ctx,
         }
 
         if (!ignore) {
-            GStringChunk *chunk = ctx->current_package->chunk;
+            GStringChunk *chunk = sctx->current_package->chunk;
 
             dep = dependency_new ();
             dep->name = g_string_chunk_insert (chunk, tmp_name);
@@ -312,9 +327,10 @@ static void
 primary_sax_start_element (void *data, const char *name, const char **attrs)
 {
     PrimarySAXContext *ctx = (PrimarySAXContext *) data;
+    SAXContext *sctx = &ctx->sctx;
 
-    if (ctx->text_buffer->len)
-        g_string_truncate (ctx->text_buffer, 0);
+    if (sctx->text_buffer->len)
+        g_string_truncate (sctx->text_buffer, 0);
 
     switch (ctx->state) {
     case PRIMARY_PARSER_TOPLEVEL:
@@ -338,89 +354,93 @@ primary_sax_start_element (void *data, const char *name, const char **attrs)
 static void
 primary_parser_package_end (PrimarySAXContext *ctx, const char *name)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
 
     g_assert (p != NULL);
 
     if (!strcmp (name, "package")) {
-        if (ctx->package_fn && !*ctx->error)
-            ctx->package_fn (p, ctx->user_data);
+        if (sctx->package_fn && !*sctx->error)
+            sctx->package_fn (p, sctx->user_data);
 
         package_free (p);
-        ctx->current_package = NULL;
+        sctx->current_package = NULL;
 
-        ctx->want_text = FALSE;
+        sctx->want_text = FALSE;
         ctx->state = PRIMARY_PARSER_TOPLEVEL;
     }
 
-    else if (ctx->text_buffer->len == 0)
+    else if (sctx->text_buffer->len == 0)
         /* Nothing interesting to do here */
         return;
 
     else if (!strcmp (name, "name"))
         p->name = g_string_chunk_insert_len (p->chunk,
-                                             ctx->text_buffer->str,
-                                             ctx->text_buffer->len);
+                                             sctx->text_buffer->str,
+                                             sctx->text_buffer->len);
     else if (!strcmp (name, "arch"))
         p->arch = g_string_chunk_insert_len (p->chunk,
-                                             ctx->text_buffer->str,
-                                             ctx->text_buffer->len);
+                                             sctx->text_buffer->str,
+                                             sctx->text_buffer->len);
     else if (!strcmp (name, "checksum"))
         p->pkgId = g_string_chunk_insert_len (p->chunk,
-                                              ctx->text_buffer->str,
-                                              ctx->text_buffer->len);
+                                              sctx->text_buffer->str,
+                                              sctx->text_buffer->len);
     else if (!strcmp (name, "summary"))
         p->summary = g_string_chunk_insert_len (p->chunk,
-                                                ctx->text_buffer->str,
-                                                ctx->text_buffer->len);
+                                                sctx->text_buffer->str,
+                                                sctx->text_buffer->len);
     else if (!strcmp (name, "description"))
         p->description = g_string_chunk_insert_len (p->chunk,
-                                                    ctx->text_buffer->str,
-                                                    ctx->text_buffer->len);
+                                                    sctx->text_buffer->str,
+                                                    sctx->text_buffer->len);
     else if (!strcmp (name, "packager"))
         p->rpm_packager = g_string_chunk_insert_len (p->chunk,
-                                                     ctx->text_buffer->str,
-                                                     ctx->text_buffer->len);
+                                                     sctx->text_buffer->str,
+                                                     sctx->text_buffer->len);
     else if (!strcmp (name, "url"))
         p->url = g_string_chunk_insert_len (p->chunk,
-                                            ctx->text_buffer->str,
-                                            ctx->text_buffer->len);
+                                            sctx->text_buffer->str,
+                                            sctx->text_buffer->len);
 }
 
 static void
 primary_parser_format_end (PrimarySAXContext *ctx, const char *name)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
 
     g_assert (p != NULL);
 
     if (!strcmp (name, "rpm:license"))
         p->rpm_license = g_string_chunk_insert_len (p->chunk,
-                                                    ctx->text_buffer->str,
-                                                    ctx->text_buffer->len);
+                                                    sctx->text_buffer->str,
+                                                    sctx->text_buffer->len);
     if (!strcmp (name, "rpm:vendor"))
         p->rpm_vendor = g_string_chunk_insert_len (p->chunk,
-                                                   ctx->text_buffer->str,
-                                                   ctx->text_buffer->len);
+                                                   sctx->text_buffer->str,
+                                                   sctx->text_buffer->len);
     if (!strcmp (name, "rpm:group"))
         p->rpm_group = g_string_chunk_insert_len (p->chunk,
-                                                  ctx->text_buffer->str,
-                                                  ctx->text_buffer->len);
+                                                  sctx->text_buffer->str,
+                                                  sctx->text_buffer->len);
     if (!strcmp (name, "rpm:buildhost"))
         p->rpm_buildhost = g_string_chunk_insert_len (p->chunk,
-                                                      ctx->text_buffer->str,
-                                                      ctx->text_buffer->len);
+                                                      sctx->text_buffer->str,
+                                                      sctx->text_buffer->len);
     if (!strcmp (name, "rpm:sourcerpm"))
         p->rpm_sourcerpm = g_string_chunk_insert_len (p->chunk,
-                                                      ctx->text_buffer->str,
-                                                      ctx->text_buffer->len);
+                                                      sctx->text_buffer->str,
+                                                      sctx->text_buffer->len);
     else if (!strcmp (name, "file")) {
         PackageFile *file = ctx->current_file != NULL ?
             ctx->current_file : package_file_new ();
 
         file->name = g_string_chunk_insert_len (p->chunk,
-                                                ctx->text_buffer->str,
-                                                ctx->text_buffer->len);
+                                                sctx->text_buffer->str,
+                                                sctx->text_buffer->len);
 
         if (!file->type)
             file->type = g_string_chunk_insert_const (p->chunk, "file");
@@ -434,7 +454,9 @@ primary_parser_format_end (PrimarySAXContext *ctx, const char *name)
 static void
 primary_parser_dep_end (PrimarySAXContext *ctx, const char *name)
 {
-    g_assert (ctx->current_package != NULL);
+    SAXContext *sctx = &ctx->sctx;
+
+    g_assert (sctx->current_package != NULL);
 
     if (strcmp (name, "rpm:entry"))
         ctx->state = PRIMARY_PARSER_FORMAT;
@@ -444,6 +466,7 @@ static void
 primary_sax_end_element (void *data, const char *name)
 {
     PrimarySAXContext *ctx = (PrimarySAXContext *) data;
+    SAXContext *sctx = &ctx->sctx;
 
     switch (ctx->state) {
     case PRIMARY_PARSER_PACKAGE:
@@ -459,20 +482,20 @@ primary_sax_end_element (void *data, const char *name)
         break;
     }
 
-    g_string_truncate (ctx->text_buffer, 0);
+    g_string_truncate (sctx->text_buffer, 0);
 }
 
 static void
-primary_sax_characters (void *data, const char *ch, int len)
+sax_characters (void *data, const char *ch, int len)
 {
-    PrimarySAXContext *ctx = (PrimarySAXContext *) data;
+    SAXContext *sctx = (SAXContext *) data;
 
-    if (ctx->want_text)
-        g_string_append_len (ctx->text_buffer, ch, len);
+    if (sctx->want_text)
+        g_string_append_len (sctx->text_buffer, ch, len);
 }
 
 static void
-primary_sax_warning (void *data, const char *msg, ...)
+sax_warning (void *data, const char *msg, ...)
 {
     va_list args;
     char *tmp;
@@ -487,17 +510,17 @@ primary_sax_warning (void *data, const char *msg, ...)
 }
 
 static void
-primary_sax_error (void *data, const char *msg, ...)
+sax_error (void *data, const char *msg, ...)
 {
-    PrimarySAXContext *ctx = (PrimarySAXContext *) data;
+    SAXContext *sctx = (SAXContext *) data;
     va_list args;
     char *tmp;
 
     va_start (args, msg);
 
     tmp = g_strdup_vprintf (msg, args);
-    g_set_error (ctx->error, YUM_PARSER_ERROR, YUM_PARSER_ERROR,
-                 "Parsing primary.xml error: %s", tmp);
+    g_set_error (sctx->error, YUM_PARSER_ERROR, YUM_PARSER_ERROR,
+                 "Parsing %s error: %s", sctx->md_type, tmp);
     g_free (tmp);
 
     va_end (args);
@@ -521,15 +544,33 @@ static xmlSAXHandler primary_sax_handler = {
     (startElementSAXFunc) primary_sax_start_element, /* startElement */
     (endElementSAXFunc) primary_sax_end_element,     /* endElement */
     NULL,      /* reference */
-    (charactersSAXFunc) primary_sax_characters,      /* characters */
+    (charactersSAXFunc) sax_characters,      /* characters */
     NULL,      /* ignorableWhitespace */
     NULL,      /* processingInstruction */
     NULL,      /* comment */
-    primary_sax_warning,      /* warning */
-    primary_sax_error,      /* error */
-    primary_sax_error,      /* fatalError */
+    sax_warning,      /* warning */
+    sax_error,      /* error */
+    sax_error,      /* fatalError */
 };
 
+void
+sax_context_init (SAXContext *sctx,
+                  const char *md_type,
+                  CountFn count_callback,
+                  PackageFn package_callback,
+                  gpointer user_data,
+                  GError **err)
+{
+    sctx->md_type = md_type;
+    sctx->error = err;
+    sctx->count_fn = count_callback;
+    sctx->package_fn = package_callback;
+    sctx->user_data = user_data;
+    sctx->current_package = NULL;
+    sctx->want_text = FALSE;
+    sctx->text_buffer = g_string_sized_new (PACKAGE_FIELD_SIZE);
+}
+
 void
 yum_xml_parse_primary (const char *filename,
                        CountFn count_callback,
@@ -538,28 +579,25 @@ yum_xml_parse_primary (const char *filename,
                        GError **err)
 {
     PrimarySAXContext ctx;
+    SAXContext *sctx = &ctx.sctx;
     int rc;
 
     ctx.state = PRIMARY_PARSER_TOPLEVEL;
-    ctx.error = err;
-    ctx.count_fn = count_callback;
-    ctx.package_fn = package_callback;
-    ctx.user_data = user_data;
-    ctx.current_package = NULL;
     ctx.current_dep_list = NULL;
     ctx.current_file = NULL;
-    ctx.want_text = FALSE;
-    ctx.text_buffer = g_string_sized_new (PACKAGE_FIELD_SIZE);
+
+    sax_context_init(sctx, "primary.xml", count_callback, package_callback,
+                     user_data, err);
 
     xmlSubstituteEntitiesDefault (1);
     rc = xmlSAXUserParseFile (&primary_sax_handler, &ctx, filename);
 
-    if (ctx.current_package) {
+    if (sctx->current_package) {
         g_warning ("Incomplete package lost");
-        package_free (ctx.current_package);
+        package_free (sctx->current_package);
     }
 
-    g_string_free (ctx.text_buffer, TRUE);
+    g_string_free (sctx->text_buffer, TRUE);
 }
 
 /*****************************************************************************/
@@ -591,18 +629,11 @@ typedef enum {
 } FilelistSAXContextState;
 
 typedef struct {
-    xmlParserCtxt *xml_context;
+    SAXContext sctx;
+
     FilelistSAXContextState state;
-    GError **error;
-    CountFn count_fn;
-    PackageFn package_fn;
-    gpointer user_data;
 
-    Package *current_package;
     PackageFile *current_file;
-
-    gboolean want_text;
-    GString *text_buffer;
 } FilelistSAXContext;
 
 static void
@@ -610,16 +641,18 @@ filelist_parser_toplevel_start (FilelistSAXContext *ctx,
                                 const char *name,
                                 const char **attrs)
 {
+    SAXContext *sctx = &ctx->sctx;
+
     if (!strcmp (name, "package")) {
-        g_assert (ctx->current_package == NULL);
+        g_assert (sctx->current_package == NULL);
 
         ctx->state = FILELIST_PARSER_PACKAGE;
 
-        ctx->current_package = package_new ();
-        parse_package (attrs, ctx->current_package);
+        sctx->current_package = package_new ();
+        parse_package (attrs, sctx->current_package);
     }
 
-    else if (ctx->count_fn && !strcmp (name, "filelists")) {
+    else if (sctx->count_fn && !strcmp (name, "filelists")) {
         int i;
         const char *attr;
         const char *value;
@@ -629,8 +662,8 @@ filelist_parser_toplevel_start (FilelistSAXContext *ctx,
             value = attrs[++i];
 
             if (!strcmp (attr, "packages")) {
-                ctx->count_fn (string_to_guint32_with_default (value, 0),
-                               ctx->user_data);
+                sctx->count_fn (string_to_guint32_with_default (value, 0),
+                                sctx->user_data);
                 break;
             }
         }
@@ -642,14 +675,16 @@ filelist_parser_package_start (FilelistSAXContext *ctx,
                                const char *name,
                                const char **attrs)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
     int i;
     const char *attr;
     const char *value;
 
     g_assert (p != NULL);
 
-    ctx->want_text = TRUE;
+    sctx->want_text = TRUE;
 
     if (!strcmp (name, "version")) {
         parse_version_info(attrs, p);
@@ -673,9 +708,10 @@ static void
 filelist_sax_start_element (void *data, const char *name, const char **attrs)
 {
     FilelistSAXContext *ctx = (FilelistSAXContext *) data;
+    SAXContext *sctx = &ctx->sctx;
 
-    if (ctx->text_buffer->len)
-        g_string_truncate (ctx->text_buffer, 0);
+    if (sctx->text_buffer->len)
+        g_string_truncate (sctx->text_buffer, 0);
 
     switch (ctx->state) {
     case FILELIST_PARSER_TOPLEVEL:
@@ -692,18 +728,20 @@ filelist_sax_start_element (void *data, const char *name, const char **attrs)
 static void
 filelist_parser_package_end (FilelistSAXContext *ctx, const char *name)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
 
     g_assert (p != NULL);
 
-    ctx->want_text = FALSE;
+    sctx->want_text = FALSE;
 
     if (!strcmp (name, "package")) {
-        if (ctx->package_fn && !*ctx->error)
-            ctx->package_fn (p, ctx->user_data);
+        if (sctx->package_fn && !*sctx->error)
+            sctx->package_fn (p, sctx->user_data);
 
         package_free (p);
-        ctx->current_package = NULL;
+        sctx->current_package = NULL;
 
         if (ctx->current_file) {
             g_free (ctx->current_file);
@@ -716,8 +754,8 @@ filelist_parser_package_end (FilelistSAXContext *ctx, const char *name)
     else if (!strcmp (name, "file")) {
         PackageFile *file = ctx->current_file;
         file->name = g_string_chunk_insert_len (p->chunk,
-                                                ctx->text_buffer->str,
-                                                ctx->text_buffer->len);
+                                                sctx->text_buffer->str,
+                                                sctx->text_buffer->len);
         if (!file->type)
             file->type = g_string_chunk_insert_const (p->chunk, "file");
 
@@ -730,6 +768,7 @@ static void
 filelist_sax_end_element (void *data, const char *name)
 {
     FilelistSAXContext *ctx = (FilelistSAXContext *) data;
+    SAXContext *sctx = &ctx->sctx;
 
     switch (ctx->state) {
     case FILELIST_PARSER_PACKAGE:
@@ -739,48 +778,7 @@ filelist_sax_end_element (void *data, const char *name)
         break;
     }
 
-    g_string_truncate (ctx->text_buffer, 0);
-}
-
-static void
-filelist_sax_characters (void *data, const char *ch, int len)
-{
-    FilelistSAXContext *ctx = (FilelistSAXContext *) data;
-
-    if (ctx->want_text)
-        g_string_append_len (ctx->text_buffer, ch, len);
-}
-
-static void
-filelist_sax_warning (void *data, const char *msg, ...)
-{
-    va_list args;
-    char *tmp;
-
-    va_start (args, msg);
-
-    tmp = g_strdup_vprintf (msg, args);
-    g_warning ("* SAX Warning: %s", tmp);
-    g_free (tmp);
-
-    va_end (args);
-}
-
-static void
-filelist_sax_error (void *data, const char *msg, ...)
-{
-    PrimarySAXContext *ctx = (PrimarySAXContext *) data;
-    va_list args;
-    char *tmp;
-
-    va_start (args, msg);
-
-    tmp = g_strdup_vprintf (msg, args);
-    g_set_error (ctx->error, YUM_PARSER_ERROR, YUM_PARSER_ERROR,
-                 "Parsing filelists.xml error: %s", tmp);
-    g_free (tmp);
-
-    va_end (args);
+    g_string_truncate (sctx->text_buffer, 0);
 }
 
 static xmlSAXHandler filelist_sax_handler = {
@@ -801,13 +799,13 @@ static xmlSAXHandler filelist_sax_handler = {
     (startElementSAXFunc) filelist_sax_start_element, /* startElement */
     (endElementSAXFunc) filelist_sax_end_element,     /* endElement */
     NULL,      /* reference */
-    (charactersSAXFunc) filelist_sax_characters,      /* characters */
+    (charactersSAXFunc) sax_characters,      /* characters */
     NULL,      /* ignorableWhitespace */
     NULL,      /* processingInstruction */
     NULL,      /* comment */
-    filelist_sax_warning,      /* warning */
-    filelist_sax_error,      /* error */
-    filelist_sax_error,      /* fatalError */
+    sax_warning,      /* warning */
+    sax_error,      /* error */
+    sax_error,      /* fatalError */
 };
 
 void
@@ -818,30 +816,28 @@ yum_xml_parse_filelists (const char *filename,
                          GError **err)
 {
     FilelistSAXContext ctx;
+    SAXContext *sctx = &ctx.sctx;
+
     int rc;
 
     ctx.state = FILELIST_PARSER_TOPLEVEL;
-    ctx.error = err;
-    ctx.count_fn = count_callback;
-    ctx.package_fn = package_callback;
-    ctx.user_data = user_data;
-    ctx.current_package = NULL;
     ctx.current_file = NULL;
-    ctx.want_text = FALSE;
-    ctx.text_buffer = g_string_sized_new (PACKAGE_FIELD_SIZE);
+    
+    sax_context_init(sctx, "filelists.xml", count_callback, package_callback,
+                     user_data, err);
 
     xmlSubstituteEntitiesDefault (1);
     rc = xmlSAXUserParseFile (&filelist_sax_handler, &ctx, filename);
 
-    if (ctx.current_package) {
+    if (sctx->current_package) {
         g_warning ("Incomplete package lost");
-        package_free (ctx.current_package);
+        package_free (sctx->current_package);
     }
 
     if (ctx.current_file)
         g_free (ctx.current_file);
 
-    g_string_free (ctx.text_buffer, TRUE);
+    g_string_free (sctx->text_buffer, TRUE);
 }
 
 /*****************************************************************************/
@@ -852,18 +848,11 @@ typedef enum {
 } OtherSAXContextState;
 
 typedef struct {
-    xmlParserCtxt *xml_context;
+    SAXContext sctx;
+
     OtherSAXContextState state;
-    GError **error;
-    CountFn count_fn;
-    PackageFn package_fn;
-    gpointer user_data;
 
-    Package *current_package;
     ChangelogEntry *current_entry;
-
-    gboolean want_text;
-    GString *text_buffer;
 } OtherSAXContext;
 
 static void
@@ -871,16 +860,18 @@ other_parser_toplevel_start (OtherSAXContext *ctx,
                              const char *name,
                              const char **attrs)
 {
+    SAXContext *sctx = &ctx->sctx;
+
     if (!strcmp (name, "package")) {
-        g_assert (ctx->current_package == NULL);
+        g_assert (sctx->current_package == NULL);
 
         ctx->state = OTHER_PARSER_PACKAGE;
 
-        ctx->current_package = package_new ();
-        parse_package (attrs, ctx->current_package);
+        sctx->current_package = package_new ();
+        parse_package (attrs, sctx->current_package);
     }
 
-    else if (ctx->count_fn && !strcmp (name, "otherdata")) {
+    else if (sctx->count_fn && !strcmp (name, "otherdata")) {
         int i;
         const char *attr;
         const char *value;
@@ -890,8 +881,8 @@ other_parser_toplevel_start (OtherSAXContext *ctx,
             value = attrs[++i];
 
             if (!strcmp (attr, "packages")) {
-                ctx->count_fn (string_to_guint32_with_default (value, 0),
-                               ctx->user_data);
+                sctx->count_fn (string_to_guint32_with_default (value, 0),
+                                sctx->user_data);
                 break;
             }
         }
@@ -903,14 +894,16 @@ other_parser_package_start (OtherSAXContext *ctx,
                             const char *name,
                             const char **attrs)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
     int i;
     const char *attr;
     const char *value;
 
     g_assert (p != NULL);
 
-    ctx->want_text = TRUE;
+    sctx->want_text = TRUE;
 
     if (!strcmp (name, "version")) {
         parse_version_info(attrs, p);
@@ -936,9 +929,10 @@ static void
 other_sax_start_element (void *data, const char *name, const char **attrs)
 {
     OtherSAXContext *ctx = (OtherSAXContext *) data;
+    SAXContext *sctx = &ctx->sctx;
 
-    if (ctx->text_buffer->len)
-        g_string_truncate (ctx->text_buffer, 0);
+    if (sctx->text_buffer->len)
+        g_string_truncate (sctx->text_buffer, 0);
 
     switch (ctx->state) {
     case OTHER_PARSER_TOPLEVEL:
@@ -955,22 +949,24 @@ other_sax_start_element (void *data, const char *name, const char **attrs)
 static void
 other_parser_package_end (OtherSAXContext *ctx, const char *name)
 {
-    Package *p = ctx->current_package;
+    SAXContext *sctx = &ctx->sctx;
+
+    Package *p = sctx->current_package;
 
     g_assert (p != NULL);
 
-    ctx->want_text = FALSE;
+    sctx->want_text = FALSE;
 
     if (!strcmp (name, "package")) {
 
         if (p->changelogs)
             p->changelogs = g_slist_reverse (p->changelogs);
 
-        if (ctx->package_fn && !*ctx->error)
-            ctx->package_fn (p, ctx->user_data);
+        if (sctx->package_fn && !*sctx->error)
+            sctx->package_fn (p, sctx->user_data);
 
         package_free (p);
-        ctx->current_package = NULL;
+        sctx->current_package = NULL;
 
         if (ctx->current_entry) {
             g_free (ctx->current_entry);
@@ -983,8 +979,8 @@ other_parser_package_end (OtherSAXContext *ctx, const char *name)
     else if (!strcmp (name, "changelog")) {
         ctx->current_entry->changelog =
             g_string_chunk_insert_len (p->chunk,
-                                       ctx->text_buffer->str,
-                                       ctx->text_buffer->len);
+                                       sctx->text_buffer->str,
+                                       sctx->text_buffer->len);
 
         p->changelogs = g_slist_prepend (p->changelogs, ctx->current_entry);
         ctx->current_entry = NULL;
@@ -995,6 +991,7 @@ static void
 other_sax_end_element (void *data, const char *name)
 {
     OtherSAXContext *ctx = (OtherSAXContext *) data;
+    SAXContext *sctx = &ctx->sctx;
 
     switch (ctx->state) {
     case OTHER_PARSER_PACKAGE:
@@ -1004,48 +1001,7 @@ other_sax_end_element (void *data, const char *name)
         break;
     }
 
-    g_string_truncate (ctx->text_buffer, 0);
-}
-
-static void
-other_sax_characters (void *data, const char *ch, int len)
-{
-    OtherSAXContext *ctx = (OtherSAXContext *) data;
-
-    if (ctx->want_text)
-        g_string_append_len (ctx->text_buffer, ch, len);
-}
-
-static void
-other_sax_warning (void *data, const char *msg, ...)
-{
-    va_list args;
-    char *tmp;
-
-    va_start (args, msg);
-
-    tmp = g_strdup_vprintf (msg, args);
-    g_warning ("* SAX Warning: %s", tmp);
-    g_free (tmp);
-
-    va_end (args);
-}
-
-static void
-other_sax_error (void *data, const char *msg, ...)
-{
-    PrimarySAXContext *ctx = (PrimarySAXContext *) data;
-    va_list args;
-    char *tmp;
-
-    va_start (args, msg);
-
-    tmp = g_strdup_vprintf (msg, args);
-    g_set_error (ctx->error, YUM_PARSER_ERROR, YUM_PARSER_ERROR,
-                 "Parsing other.xml error: %s", tmp);
-    g_free (tmp);
-
-    va_end (args);
+    g_string_truncate (sctx->text_buffer, 0);
 }
 
 static xmlSAXHandler other_sax_handler = {
@@ -1066,13 +1022,13 @@ static xmlSAXHandler other_sax_handler = {
     (startElementSAXFunc) other_sax_start_element, /* startElement */
     (endElementSAXFunc) other_sax_end_element,     /* endElement */
     NULL,      /* reference */
-    (charactersSAXFunc) other_sax_characters,      /* characters */
+    (charactersSAXFunc) sax_characters,      /* characters */
     NULL,      /* ignorableWhitespace */
     NULL,      /* processingInstruction */
     NULL,      /* comment */
-    other_sax_warning,      /* warning */
-    other_sax_error,      /* error */
-    other_sax_error,      /* fatalError */
+    sax_warning,      /* warning */
+    sax_error,      /* error */
+    sax_error,      /* fatalError */
 };
 
 void
@@ -1083,28 +1039,26 @@ yum_xml_parse_other (const char *filename,
                      GError **err)
 {
     OtherSAXContext ctx;
+    SAXContext *sctx = &ctx.sctx;
+
     int rc;
 
     ctx.state = OTHER_PARSER_TOPLEVEL;
-    ctx.error = err;
-    ctx.count_fn = count_callback;
-    ctx.package_fn = package_callback;
-    ctx.user_data = user_data;
-    ctx.current_package = NULL;
     ctx.current_entry = NULL;
-    ctx.want_text = FALSE;
-    ctx.text_buffer = g_string_sized_new (PACKAGE_FIELD_SIZE);
+    
+    sax_context_init(sctx, "other.xml", count_callback, package_callback,
+                     user_data, err);
 
     xmlSubstituteEntitiesDefault (1);
     rc = xmlSAXUserParseFile (&other_sax_handler, &ctx, filename);
 
-    if (ctx.current_package) {
+    if (sctx->current_package) {
         g_warning ("Incomplete package lost");
-        package_free (ctx.current_package);
+        package_free (sctx->current_package);
     }
 
     if (ctx.current_entry)
         g_free (ctx.current_entry);
 
-    g_string_free (ctx.text_buffer, TRUE);
+    g_string_free (sctx->text_buffer, TRUE);
 }
